
#include <afxdisp.h>
#include <gdiplus.h>
#include <stdio.h>

void checkForFiles(char** argv);
wchar_t* convertToUnicode(char* val);

using namespace Gdiplus;

void main(int argc, char** argv)
{
  if (argc < 3)
  {
    fprintf(stderr, "Usage: emf2wmf <input-file> <output-file>\n");
    exit(1);
  }

  checkForFiles(argv);

  GdiplusStartupInput m_gdiplusStartupInput;
  ULONG_PTR m_pGdiToken;
  GdiplusStartup(&m_pGdiToken,&m_gdiplusStartupInput,NULL);  //gdi+ init

  // Construct a Metafile object from an existing EMF disk file.
  wchar_t* inFilename = convertToUnicode(argv[1]);
  Metafile* myMetafile = new Metafile(inFilename);

  // Get a handle to the EMF metafile.
  HENHMETAFILE hEmf = myMetafile->GetHENHMETAFILE();

  // Determine the size of the buffer that will receive the converted records.
  UINT size = Metafile::EmfToWmfBits(
    hEmf, 
    0, 
    NULL,
    MM_ANISOTROPIC,
    EmfToWmfBitsFlagsEmbedEmf);

  // Allocate a buffer to receive the converted records.
  BYTE* buffer = new BYTE[size];

  // Convert the EMF records to WMF records.
  INT convertedSize = Metafile::EmfToWmfBits(
    hEmf, 
    size,
    buffer, 
    MM_ANISOTROPIC,
    EmfToWmfBitsFlagsEmbedEmf);

  // Get a handle to the converted metafile.
  HMETAFILE hmf = SetMetaFileBitsEx(size, buffer);

  // Write the WMF metafile to a disk file.
  CopyMetaFile(hmf, argv[2]);

  DeleteMetaFile(hmf);
  DeleteEnhMetaFile(hEmf);
  delete[] buffer;
  delete[] inFilename;

  GdiplusShutdown(m_pGdiToken); //gdi+ end session
}

#include <sys/types.h>
#include <sys/stat.h>

void checkForFiles(char** argv)
{
  struct _stat stat;
  if (_stat(argv[1], &stat) == -1)
  {
    fprintf(stderr, "Input file '%s' does not exist!\n", argv[1]);
    exit(1);
  }

  if (_stat(argv[2], &stat) != -1)
  {
    fprintf(stderr, "Warning.  Output file '%s' already exists.  It will be overwritten.\n", argv[2]);
  }
}

wchar_t* convertToUnicode(char* val)
{
  size_t len = strlen(val)*3;  // make sure we have enough space
  wchar_t* res = (wchar_t*) new char[len];

  if (res == NULL) return res;
  
  MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, val, -1, res, (int) len);
  return res;
}
